@@ -1,11 +1,17 @@
-dofile(input.find_file(instance,"luat-log.lua"))
+if not modules then modules = { } end modules ['mtx-context'] = {
+    version   = 1.001,
+    comment   = "companion to mtxrun.lua",
+    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
+    copyright = "PRAGMA ADE / ConTeXt Development Team",
+    license   = "see context related readme files"
+}
 
 texmf.instance = instance -- we need to get rid of this / maybe current instance in global table
 
 scripts         = scripts         or { }
 scripts.context = scripts.context or { }
 
--- l-file
+-- l-file / todo
 
 function file.needsupdate(oldfile,newfile)
     return true
@@ -34,9 +40,9 @@ function input.locate_format(name) -- move this to core / luat-xxx
     if fmtname ~= "" then
         barename = fmtname:gsub("%.%a+$","")
         local luaname, lucname = barename .. ".lua", barename .. ".luc"
-        if io.exists(lucname) then
+        if lfs.isfile(lucname) then
             return barename, luaname
-        elseif io.exists(luaname) then
+        elseif lfs.isfile(luaname) then
             return barename, luaname
         end
     end
@@ -68,7 +74,7 @@ do
         local attributes = e.at
         if str and attributes then
             if attributes['method'] then
-                str = ctsrunner.filtered(str,attributes['method'])
+                str = ctxrunner.filtered(str,attributes['method'])
             end
             if str == "" and attributes['default'] then
                 str = attributes['default']
@@ -190,12 +196,12 @@ do
         -- mtxrun should resolve kpse: and file:
 
         local usedname = ctxdata.ctxname
-        local found    = io.exists(usedname)
+        local found    = lfs.isfile(usedname)
 
         if not found then
-            for _, path in (ctsrunner.locations) do
+            for _, path in pairs(ctxdata.locations) do
                 local fullname = file.join(path,ctxdata.ctxname)
-                if io.exists(fullname) then
+                if lfs.isfile(fullname) then
                     usedname, found = fullname, true
                     break
                 end
@@ -222,12 +228,12 @@ do
 
         ctxdata.variables['job'] = ctxdata.jobname
 
-        ctxdata.flags        = xml.all_texts(ctxdata.xmldata,"/ctx:job/ctx:flags/ctx:flag",true)
-        ctxdata.environments = xml.all_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:environment",true)
-        ctxdata.modules      = xml.all_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:module",true)
-        ctxdata.filters      = xml.all_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:filter",true)
-        ctxdata.modes        = xml.all_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:mode",true)
-        ctxdata.messages     = xml.all_texts(ctxdata.xmldata,"ctx:message",true)
+        ctxdata.flags        = xml.collect_texts(ctxdata.xmldata,"/ctx:job/ctx:flags/ctx:flag",true)
+        ctxdata.environments = xml.collect_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:environment",true)
+        ctxdata.modules      = xml.collect_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:module",true)
+        ctxdata.filters      = xml.collect_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:filter",true)
+        ctxdata.modes        = xml.collect_texts(ctxdata.xmldata,"/ctx:job/ctx:process/ctx:resources/ctx:mode",true)
+        ctxdata.messages     = xml.collect_texts(ctxdata.xmldata,"ctx:message",true)
 
         ctxdata.flags = ctxrunner.reflag(ctxdata.flags)
 
@@ -329,7 +335,7 @@ do
                                         -- potential optimization: when mtxrun run internal
                                         command = ctxrunner.justtext(command) -- command is still xml element here
                                         input.report("command",command)
-                                        local result = os.execute(command)
+                                        local result = os.spawn(command)
                                         if result > 0 then
                                             input.report("error, return code",result)
                                         end
@@ -338,7 +344,7 @@ do
                                         end
                                     end
                                 end
-                                if io.exists(newfile) then
+                                if lfs.isfile(newfile) then
                                     file.syncmtimes(oldfile,newfile)
                                     ctxdata.prepfiles[oldfile] = true
                                 else
@@ -347,7 +353,7 @@ do
                                 end
                             else
                                 input.report("old file needs no preprocessing")
-                                ctxdata.prepfiles[oldfile] = io.exists(newfile)
+                                ctxdata.prepfiles[oldfile] = lfs.isfile(newfile)
                             end
                         end
                     end
@@ -399,10 +405,10 @@ function scripts.context.multipass.makeoptionfile(jobname,ctxdata)
     local f = io.open(jobname..".top","w")
     if f then
         local finalrun, kindofrun, currentrun = false, 0, 0
---~         local function someflag(flag)
---~             return (ctxdata and ctxdata.flags[flag]) or environment.argument(flag)
---~         end
-        local someflag = environment.argument
+        local function someflag(flag)
+            return (ctxdata and ctxdata.flags[flag]) or environment.argument(flag)
+        end
+--~         local someflag = environment.argument
         local function setvalue(flag,format,hash,default)
             local a = someflag(flag) or default
             if a and a ~= "" then
@@ -416,14 +422,14 @@ function scripts.context.multipass.makeoptionfile(jobname,ctxdata)
             end
         end
         local function setvalues(flag,format)
-            if type(flag) == "table" then
-                for _, v in pairs(flag) do
+            if type(flag) == "table"  then
+                for k, v in pairs(flag) do
                     f:write(format:format(v),"\n")
                 end
             else
                 local a = someflag(flag)
                 if a and a ~= "" then
-                    for _, v in a:gmatch("%s*([^,]+)") do
+                    for v in a:gmatch("%s*([^,]+)") do
                         f:write(format:format(v),"\n")
                     end
                 end
@@ -468,9 +474,11 @@ function scripts.context.multipass.makeoptionfile(jobname,ctxdata)
         setvalues("usemodules"   , "\\usemodule[%s]")
         setvalues("environments" , "\\environment %s ")
         -- ctx stuff
-        setvalues(ctxdata.modes,        "\\enablemode[%s]")
-        setvalues(ctxdata.modules,      "\\usemodule[%s]")
-        setvalues(ctxdata.environments, "\\environment %s ")
+        if ctxdata then
+            setvalues(ctxdata.modes,        "\\enablemode[%s]")
+            setvalues(ctxdata.modules,      "\\usemodule[%s]")
+            setvalues(ctxdata.environments, "\\environment %s ")
+        end
         -- done
         setalways(                 "\\protect")
         setalways(                 "\\endinput")
@@ -488,20 +496,38 @@ function scripts.context.multipass.copytuifile(jobname)
         g:write("% traditional utility file, only commands written by mtxrun/context\n%\n")
         for line in f:lines() do
             if line:find("^c ") then
-                g:write((line:gsub("^c ","")),"\n")
+                g:write((line:gsub("^c ","")),"%\n")
             end
         end
+        g:write("\\endinput\n")
         f:close()
         g:close()
     end
 end
 
-function scripts.context.run(ctxdata)
-    -- todo: interface
-for k,v in pairs(ctxdata.flags) do
-    environment.setargument(k,v)
-end
+scripts.context.xmlsuffixes = table.tohash {
+    "xml",
+}
 
+function scripts.context.run(ctxdata)
+    local function makestub(format,filename)
+        local stubname = file.replacesuffix(file.basename(filename),'run')
+        local f = io.open(stubname,'w')
+        if f then
+            f:write("\\starttext\n")
+            f:write(string.format(format,filename),"\n")
+            f:write("\\stoptext\n")
+            f:close()
+            filename = stubname
+        end
+        return filename
+    end
+    if ctxdata then
+        -- todo: interface
+        for k,v in pairs(ctxdata.flags) do
+            environment.setargument(k,v)
+        end
+    end
     local files = environment.files
     if #files > 0 then
         input.identify_cnf(instance)
@@ -513,20 +539,18 @@ end
             for _, filename in ipairs(files) do
                 local basename, pathname = file.basename(filename), file.dirname(filename)
                 local jobname = file.removesuffix(basename)
-                if pathname ~= "" and pathname ~= "." then
+                if pathname == "" then
                     filename = "./" .. filename
                 end
                 -- also other stubs
-                if environment.argument("forcexml") then
-                    local stubname = file.replacesuffix(file.basename(filename),'run')
-                    local f = io.open(stubname,'w')
-                    if f then
-                        f:write("\\starttext\n")
-                        f:write(string.format("\\processXMLfilegrouped{%s}\n",filename))
-                        f:write("\\stoptext\n")
-                        f:close()
-                        filename = stubname
-                    end
+                if environment.argument("forcexml") or scripts.context.xmlsuffixes[file.extname(filename) or "?"] then -- mkii
+                    filename = makestub("\\processXMLfilegrouped{%s}",filename)
+                elseif environment.argument("processxml") then -- mkiv
+                    filename = makestub("\\xmlprocess{%s}",filename)
+                end
+                --
+                if environment.argument("autopdf") then
+                    os.spawn(string.format('pdfclose --file "%s" 2>&1', file.replacesuffix(filename,"pdf")))
                 end
                 --
                 local command = "luatex --fmt=" .. string.quote(formatfile) .. " --lua=" .. string.quote(scriptfile) .. " " .. string.quote(filename)
@@ -534,7 +558,7 @@ end
                 scripts.context.multipass.makeoptionfile(jobname,ctxdata)
                 for i=1, scripts.context.multipass.nofruns do
                     input.report(string.format("run %s: %s",i,command))
-                    local returncode = os.execute(command)
+                    local returncode = os.spawn(command)
                     input.report("return code: " .. returncode)
                     if returncode > 0 then
                         input.report("fatal error, run aborted")
@@ -550,22 +574,37 @@ end
                         end
                     end
                 end
+                --
+                -- todo: result
+                --
+                if environment.argument("autopdf") then
+                    os.spawn(string.format('pdfopen --file "%s" 2>&1', file.replacesuffix(filename,"pdf")))
+                end
+                --
             end
         else
-            input.error("no format found with name " .. formatname)
+            input.verbose = true
+            input.report("error", "no format found with name " .. formatname)
         end
     end
 end
 
 function scripts.context.make()
-    -- hack, should also be a shared function
-    for _, name in ipairs( { "cont-en", "cont-nl", "mptopdf" } ) do
+    local list = (environment.files[1] and environment.files) or { "cont-en", "cont-nl", "mptopdf" }
+    for _, name in ipairs(list) do
         local command = "luatools --make --compile " .. name
         input.report("running command: " .. command)
-        os.execute(command)
+        os.spawn(command)
     end
 end
 
+function scripts.context.generate()
+    -- hack, should also be a shared function
+    local command = "luatools --generate "
+    input.report("running command: " .. command)
+    os.spawn(command)
+end
+
 function scripts.context.ctx()
     local ctxdata = ctxrunner.new()
     ctxdata.jobname = environment.files[1]
@@ -573,26 +612,110 @@ function scripts.context.ctx()
     scripts.context.run(ctxdata)
 end
 
+function scripts.context.version()
+    local name = input.find_file(instance,"context.tex")
+    if name ~= "" then
+        input.report(string.format("main context file: %s",name))
+        local data = io.loaddata(name)
+        if data then
+            local version = data:match("\\edef\\contextversion{(.-)}")
+            if version then
+                input.report(string.format("current version  : %s",version))
+            else
+                input.report("context version: unknown, no timestamp found")
+            end
+        else
+            input.report("context version: unknown, load error")
+        end
+    else
+        input.report("main context file: unknown, 'context.tex' not found")
+    end
+end
+
+function scripts.context.touch()
+    if environment.argument("expert") then
+        local function touch(name,pattern)
+            local name = input.find_file(instance,name)
+            local olddata = io.loaddata(name)
+            if olddata then
+                local oldversion, newversion = "", os.date("%Y.%M.%d %H:%m")
+                local newdata, ok = olddata:gsub(pattern,function(pre,mid,post)
+                    oldversion = mid
+                    return pre .. newversion .. post
+                end)
+                if ok > 0 then
+                    local backup = file.replacesuffix(name,"tmp")
+                    os.remove(backup)
+                    os.rename(name,backup)
+                    io.savedata(name,newdata)
+                    return true, oldversion, newversion, name
+                else
+                    return false
+                end
+            end
+        end
+        local done, oldversion, newversion, foundname = touch("context.tex", "(\\edef\\contextversion{)(.-)(})")
+        if done then
+            input.report(string.format("old version : %s", oldversion))
+            input.report(string.format("new version : %s", newversion))
+            input.report(string.format("touched file: %s", foundname))
+            local ok, _, _, foundname = touch("cont-new.tex", "(\\newcontextversion{)(.-)(})")
+            if ok then
+                input.report(string.format("touched file: %s", foundname))
+            end
+        end
+    end
+end
+
+function scripts.context.timed(action)
+    input.starttiming(scripts.context)
+    action()
+    input.stoptiming(scripts.context)
+    input.report("total runtime: " .. input.elapsedtime(scripts.context))
+end
+
 banner = banner .. " | context tools "
 
 messages.help = [[
---run                 process (one or more) files
---make                generate formats
+--run                 process (one or more) files (default action)
+--make                create context formats formats
+--generate            generate file database etc.
 --ctx=name            use ctx file
+--version             report installed context version
+--autopdf             open pdf file afterwards
+
+--expert              expert options
+]]
+
+messages.expert = [[
+expert options: also provide --expert
+
+--touch               update context version (remake needed afterwards)
 ]]
 
 input.verbose = true
-input.start_timing(scripts.context)
+
+if environment.argument("once") then
+    scripts.context.multipass.nofruns = 1
+end
 
 if environment.argument("run") then
-    scripts.context.run()
+    scripts.context.timed(scripts.context.run)
 elseif environment.argument("make") then
-    scripts.context.make()
+    scripts.context.timed(scripts.context.make)
 elseif environment.argument("ctx") then
-    scripts.context.ctx()
+    scripts.context.timed(scripts.context.ctx)
+elseif environment.argument("version") then
+    scripts.context.version()
+elseif environment.argument("touch") then
+    scripts.context.touch()
+elseif environment.argument("help") then
+    input.help(banner,messages.help)
+elseif environment.argument("expert") then
+    input.help(banner,messages.expert)
+elseif environment.files[1] then
+    scripts.context.timed(scripts.context.run)
 else
     input.help(banner,messages.help)
 end
 
-input.stop_timing(scripts.context)
-input.report("total runtime: " .. input.elapsedtime(scripts.context))
