@@ -90,29 +90,12 @@ class TEX
     @@luafiles     = "luafiles.tmp"
     @@luatarget    = "lua/context"
 
-    # we now drop pdfetex definitely
-
-    # ENV['PATH'].split(File::PATH_SEPARATOR).each do |p|
-        # if System.unix? then
-            # pp, pe = "#{p}/pdftex"    , "#{p}/pdfetex"
-        # else
-            # pp, pe = "#{p}/pdftex.exe", "#{p}/pdfetex.exe"
-        # end
-        # if    FileTest.file?(pe) then # we assume no update
-            # @@pdftex = 'pdfetex'
-            # break
-        # elsif FileTest.file?(pp) then # we assume an update
-            # @@pdftex = 'pdftex'
-            # break
-        # end
-    # end
-
-    # ['etex','pdfetex','standard']                .each do |e| @@texengines[e] = @@pdftex    end
-    # ['tex','pdftex']                             .each do |e| @@texengines[e] = 'pdftex'    end
+    @@platformslash = if System.unix? then "\\\\" else "\\" end
 
     ['tex','etex','pdftex','pdfetex','standard']   .each do |e| @@texengines[e] = 'pdftex'    end
     ['aleph','omega']                              .each do |e| @@texengines[e] = 'aleph'     end
     ['xetex']                                      .each do |e| @@texengines[e] = 'xetex'     end
+    ['petex']                                      .each do |e| @@texengines[e] = 'petex'     end
     ['luatex']                                     .each do |e| @@texengines[e] = 'luatex'    end
 
     ['metapost','mpost', 'standard']               .each do |e| @@mpsengines[e] = 'mpost'     end
@@ -120,6 +103,8 @@ class TEX
     ['pdfetex','pdftex','pdf','pdftex','standard'] .each do |b| @@backends[b]   = 'pdftex'    end
     ['dvipdfmx','dvipdfm','dpx','dpm']             .each do |b| @@backends[b]   = 'dvipdfmx'  end
     ['xetex','xtx']                                .each do |b| @@backends[b]   = 'xetex'     end
+    ['petex']                                      .each do |b| @@backends[b]   = 'dvipdfmx'  end
+    ['aleph']                                      .each do |b| @@backends[b]   = 'dvipdfmx'  end
     ['dvips','ps','dvi']                           .each do |b| @@backends[b]   = 'dvips'     end
     ['dvipsone']                                   .each do |b| @@backends[b]   = 'dvipsone'  end
     ['acrobat','adobe','distiller']                .each do |b| @@backends[b]   = 'acrobat'   end
@@ -127,7 +112,7 @@ class TEX
 
     ['tex','standard']                             .each do |b| @@mappaths[b]   = 'dvips'     end
     ['pdftex','pdfetex']                           .each do |b| @@mappaths[b]   = 'pdftex'    end
-    ['aleph','omega','xetex']                      .each do |b| @@mappaths[b]   = 'dvipdfm'   end
+    ['aleph','omega','xetex','petex']              .each do |b| @@mappaths[b]   = 'dvipdfm'   end
     ['dvipdfm', 'dvipdfmx', 'xdvipdfmx']           .each do |b| @@mappaths[b]   = 'dvipdfm'   end
     ['xdv','xdv2pdf']                              .each do |b| @@mappaths[b]   = 'dvips'     end
 
@@ -149,9 +134,7 @@ class TEX
     ['plain','mpost']                              .each do |f| @@mpsformats[f] = 'plain'     end
     ['metafun','context','standard']               .each do |f| @@mpsformats[f] = 'metafun'   end
 
-    # no 'standard' progname ! / beware, when using texexec we always use the context/metafun values
-
-    ['pdftex','pdfetex','aleph','omega',
+    ['pdftex','pdfetex','aleph','omega','petex',
      'xetex','luatex']                             .each do |p| @@prognames[p]  = 'context'   end
     ['mpost']                                      .each do |p| @@prognames[p]  = 'metafun'   end
     ['latex','pdflatex']                           .each do |p| @@prognames[p]  = 'latex'     end
@@ -164,17 +147,18 @@ class TEX
     ['plain','default','standard']                 .each do |f| @@mpsmethods[f] = 'plain'     end
     ['metafun']                                    .each do |f| @@mpsmethods[f] = 'metafun'   end
 
-    @@texmakestr['plain'] = "\\dump"
-    @@mpsmakestr['plain'] = "\\dump"
+    @@texmakestr['plain'] = @@platformslash + "dump"
+    @@mpsmakestr['plain'] = @@platformslash + "dump"
 
     ['cont-en','cont-nl','cont-de','cont-it',
-     'cont-fr','cont-cz','cont-ro','cont-uk']      .each do |f| @@texprocstr[f] = "\\emergencyend"  end
+     'cont-fr','cont-cz','cont-ro','cont-uk']      .each do |f| @@texprocstr[f] = @@platformslash + "emergencyend"  end
 
     @@runoptions['aleph']   = ['--8bit']
     @@runoptions['luatex']  = ['--file-line-error']
     @@runoptions['mpost']   = ['--8bit']
     @@runoptions['pdfetex'] = ['--8bit']           # obsolete
     @@runoptions['pdftex']  = ['--8bit']           # pdftex is now pdfetex
+  # @@runoptions['petex']   = []
     @@runoptions['xetex']   = ['--8bit','-output-driver="xdvipdfmx -E -d 4 -V 5"']
 
     @@tcxflag['aleph']   = true
@@ -182,6 +166,7 @@ class TEX
     @@tcxflag['mpost']   = true
     @@tcxflag['pdfetex'] = true
     @@tcxflag['pdftex']  = true
+    @@tcxflag['petex']   = false
     @@tcxflag['xetex']   = false
 
     @@draftoptions['pdftex'] = ['--draftmode']
@@ -592,7 +577,7 @@ class TEX
         if data = (IO.readlines(@@luafiles) rescue nil) then
             report("compiling lua files (using #{File.expand_path(@@luafiles)})")
             begin
-                Dir.makedirs(@@luatarget) rescue false
+                File.makedirs(@@luatarget) rescue false
                 data.each do |line|
                     luafile = line.chomp
                     lucfile = File.basename(luafile).gsub(/\..*?$/,'') + ".luc"
@@ -945,7 +930,8 @@ end
                 case str.chomp
                     when /^\%/o then
                         # next
-                    when /\\(starttekst|stoptekst|startonderdeel|startdocument|startoverzicht)/o then
+                #   when /\\(starttekst|stoptekst|startonderdeel|startdocument|startoverzicht)/o then
+                    when /\\(starttekst|stoptekst|startonderdeel|startoverzicht)/o then
                         setvariable('texformats','nl') ; break
                     when /\\(stelle|verwende|umgebung|benutze)/o then
                         setvariable('texformats','de') ; break
@@ -1885,7 +1871,7 @@ end
 
         if globalfile || FileTest.file?(rawname) then
 
-            if not dummyfile and not globalfile then
+            if not dummyfile and not globalfile and not forcexml then
                 scantexpreamble(rawname)
                 scantexcontent(rawname) if getvariable('texformats').standard?
             end
@@ -2072,7 +2058,7 @@ end
         setvariable('mp.line','')
         setvariable('mp.error','')
         if mpdata = File.silentread(mpname) then
-            mpdata.gsub!(/^\#.*\n/o,'')
+            mpdata.gsub!(/^\%.*\n/o,'')
             File.silentrename(mpname,mpcopy)
             texfound = mergebe || (mpdata =~ /btex .*? etex/mo)
             if mp = openedfile(mpname) then
