@@ -187,7 +187,14 @@ class TeXUtil
         # shortcut("\\\"e", 'ediaeresis')
         # shortcut("\\\'o", 'oacute')
 
+        def hextoutf(str)
+            str.gsub(/^(0x[A-F\d]+)$/) do
+                [$1.hex()].pack("U")
+            end
+        end
+
         def shortcut(from,to)
+            from = hextoutf(from)
             replacer(from,to)
             expander(to)
         end
@@ -706,8 +713,8 @@ class TeXUtil
                                     elsif alpha == @@specialsymbol then
                                         character = @@specialbanner
                                     elsif alpha.length > 1 then
-                                        # character = "\\getvalue\{#{alpha}\}%"
-                                        character = "\\#{alpha}%"
+                                        # character = "\\getvalue\{#{alpha}\}"
+                                        character = "\\#{alpha}"
                                     else
                                         character = "\\unknown"
                                     end
