@@ -19,7 +19,7 @@ away.</p>
 
 fonts                      = fonts     or { }
 fonts.afm                  = fonts.afm or { }
-fonts.afm.version          = 1.13 -- incrementing this number one up will force a re-cache
+fonts.afm.version          = 1.26 -- incrementing this number one up will force a re-cache
 fonts.afm.syncspace        = true -- when true, nicer stretch values
 fonts.afm.enhance_data     = true -- best leave this set to true
 fonts.afm.trace_features   = false
@@ -35,8 +35,62 @@ fonts.afm.cache            = containers.define("fonts", "afm", fonts.afm.version
 built in <l n='tfm'/> and <l n='otf'/> reader.</p>
 --ldx]]--
 
+--~ Comment FONTIDENTIFIER LMMATHSYMBOLS10
+--~ Comment CODINGSCHEME TEX MATH SYMBOLS
+--~ Comment DESIGNSIZE 10.0 pt
+--~ Comment CHECKSUM O 4261307036
+--~ Comment SPACE 0 plus 0 minus 0
+--~ Comment QUAD 1000
+--~ Comment EXTRASPACE 0
+--~ Comment NUM 676.508 393.732 443.731
+--~ Comment DENOM 685.951 344.841
+--~ Comment SUP 412.892 362.892 288.889
+--~ Comment SUB 150 247.217
+--~ Comment SUPDROP 386.108
+--~ Comment SUBDROP 50
+--~ Comment DELIM 2390 1010
+--~ Comment AXISHEIGHT 250
+
 do
 
+    local c = lpeg.P("Comment")
+    local s = lpeg.S(" \t")
+    local l = lpeg.S("\n\r")
+    local w = lpeg.C((1 - l)^1)
+    local n = lpeg.C((lpeg.R("09") + lpeg.S("."))^1) / tonumber * s^0
+
+    local fd = { }
+
+    local pattern = ( c * s^1 * (
+        ("CODINGSCHEME" * s^1 * w                            ) / function(a)                                      end +
+        ("DESIGNSIZE"   * s^1 * n * w                        ) / function(a)     fd[ 1]                 = a       end +
+        ("CHECKSUM"     * s^1 * n * w                        ) / function(a)     fd[ 2]                 = a       end +
+        ("SPACE"        * s^1 * n * "plus"  * n * "minus" * n) / function(a,b,c) fd[ 3], fd[ 4], fd[ 5] = a, b, c end +
+        ("QUAD"         * s^1 * n                            ) / function(a)     fd[ 6]                 = a       end +
+        ("EXTRASPACE"   * s^1 * n                            ) / function(a)     fd[ 7]                 = a       end +
+        ("NUM"          * s^1 * n * n * n                    ) / function(a,b,c) fd[ 8], fd[ 9], fd[10] = a, b, c end +
+        ("DENOM"        * s^1 * n * n                        ) / function(a,b  ) fd[11], fd[12]         = a, b    end +
+        ("SUP"          * s^1 * n * n * n                    ) / function(a,b,c) fd[13], fd[14], fd[15] = a, b, c end +
+        ("SUB"          * s^1 * n * n                        ) / function(a,b)   fd[16], fd[17]         = a, b    end +
+        ("SUPDROP"      * s^1 * n                            ) / function(a)     fd[18]                 = a       end +
+        ("SUBDROP"      * s^1 * n                            ) / function(a)     fd[19]                 = a       end +
+        ("DELIM"        * s^1 * n * n                        ) / function(a,b)   fd[20], fd[21]         = a, b    end +
+        ("AXISHEIGHT"   * s^1 * n                            ) / function(a)     fd[22]                 = a       end +
+        (1-l)^0
+    ) + (1-c)^1)^0
+
+    function fonts.afm.scan_comment(str)
+        fd = { }
+        pattern:match(str)
+        return fd
+    end
+
+end
+
+do
+
+    -- On a rainy day I will rewrite this in lpeg ...
+
     local keys = { }
 
     function keys.FontName    (data,line) data.fullname     = line:strip() end
@@ -70,7 +124,7 @@ do
                 end
                 chr.index = ind
             elseif k == 'WX' then
-                chr.wx = v
+                chr.width = v
             elseif k == 'N'  then
                 str = v
             elseif k == 'B'  then
@@ -119,32 +173,32 @@ do
                     local glyphs = pfbdata.glyphs
                     if glyphs then
                         if trace then
-                            logs.report("define font", string.format("getting index data from %s",pfbname))
+                            logs.report("load afm", string.format("getting index data from %s",pfbname))
                         end
                         -- local offset = (glyphs[0] and glyphs[0] != .notdef) or 0
-                        for index, glyph in pairs(glyphs) do -- ipairs? offset
+                        for index, glyph in pairs(glyphs) do
                             local name = glyph.name
                             if name then
                                 local char = characters[name]
                                 if char then
                                     if trace then
-                                        logs.report("define font", string.format("glyph %s has index %s",name,index))
+                                        logs.report("load afm", string.format("glyph %s has index %s",name,index))
                                     end
                                     char.index = index
                                 end
                             end
                         end
                     elseif trace then
-                        logs.report("define font", string.format("no glyph data in pfb file %s",pfbname))
+                        logs.report("load afm", string.format("no glyph data in pfb file %s",pfbname))
                     end
                 elseif trace then
-                    logs.report("define font", string.format("no data in pfb file %s",pfbname))
+                    logs.report("load afm", string.format("no data in pfb file %s",pfbname))
                 end
             elseif trace then
-                logs.report("define font", string.format("invalid pfb file %s",pfbname))
+                logs.report("load afm", string.format("invalid pfb file %s",pfbname))
             end
         elseif trace then
-            logs.report("define font", string.format("no pfb file for %s",filename))
+            logs.report("load afm", string.format("no pfb file for %s",filename))
         end
     end
 
@@ -159,29 +213,33 @@ do
             }
             afmblob = afmblob:gsub("StartCharMetrics(.-)EndCharMetrics", function(charmetrics)
                 if fonts.trace then
-                    logs.report("define font", "loading char metrics")
+                    logs.report("load afm", "loading char metrics")
                 end
                 get_charmetrics(data,charmetrics,vector)
                 return ""
             end)
             afmblob = afmblob:gsub("StartKernPairs(.-)EndKernPairs", function(kernpairs)
                 if fonts.trace then
-                    logs.report("define font", "loading kern pairs")
+                    logs.report("load afm", "loading kern pairs")
                 end
                 get_kernpairs(data,kernpairs)
                 return ""
             end)
             afmblob = afmblob:gsub("StartFontMetrics%s+([%d%.]+)(.-)EndFontMetrics", function(version,fontmetrics)
                 if fonts.trace then
-                    logs.report("define font", "loading variables")
+                    logs.report("load afm", "loading variables")
                 end
                 data.afmversion = version
                 get_variables(data,fontmetrics)
+                data.fontdimens = fonts.afm.scan_comment(fontmetrics) -- todo: all lpeg, no time now
                 return ""
             end)
             get_indexes(data,filename)
             return data
         else
+            if fonts.trace then
+                logs.report("load afm", "no valid afm file " .. filename)
+            end
             return nil
         end
     end
@@ -196,7 +254,11 @@ way we can set them faster when defining a font.</p>
 
 function fonts.afm.load(filename)
     local name = file.removesuffix(filename)
-    local data = containers.read(fonts.afm.cache,name)
+    local data = containers.read(fonts.afm.cache(),name)
+    local size = lfs.attributes(name,"size") or 0
+    if data and data.size ~= size then
+        data = nil
+    end
     if not data then
         local foundname = input.find_file(texmf.instance,filename,'afm')
         if foundname and foundname ~= "" then
@@ -208,7 +270,10 @@ function fonts.afm.load(filename)
                     fonts.afm.add_ligatures(data,'texligatures') -- easier this way
                     fonts.afm.add_kerns(data) -- faster this way
                 end
-                data = containers.write(fonts.afm.cache, name, data)
+                logs.report("load afm","file size: " .. size)
+                data.size = size
+                logs.report("load afm","saving: in cache")
+                data = containers.write(fonts.afm.cache(), name, data)
             end
         end
     end
@@ -216,12 +281,18 @@ function fonts.afm.load(filename)
 end
 
 function fonts.afm.unify(data, filename)
-    local unicode, private, unicodes = containers.content(fonts.enc.cache,'unicode').hash, 0x0F0000, { }
+--~     local unicode, unicodes, private  = fonts.enc.load('unicode').hash, { }, 0x0F0000
+    local unicode, unicodes, private  = fonts.enc.load('unicode').hash, { }, fonts.private
     for name, blob in pairs(data.characters) do
-        local code = unicode[name]
+        local code = unicode[name] -- or characters.name_to_unicode[name]
         if not code then
-            code = private
-            private = private + 1
+            local u = name:match("^uni(%x+)$")
+            code = u and tonumber(u,16)
+            if not code then
+                code = private
+                private = private + 1
+                logs.report("afm glyph", string.format("assigning private slot 0x%04X for unknown glyph name %s", code, name))
+            end
         end
         blob.unicode = code
         unicodes[name] = code
@@ -246,9 +317,10 @@ function fonts.afm.add_ligatures(afmdata,ligatures)
             for _, b in pairs(v) do
                 two, three = b[1], b[2]
                 if two and three and chars[two] and chars[three] then
-                    if one[ligatures] then
-                        if not one.ligatures[two] then
-                            one[ligatures][two] = three
+                    local ol = one[ligatures]
+                    if ol then
+                        if not ol[two] then -- was one.ligatures ... bug
+                            ol[two] = three
                         end
                     else
                         one[ligatures] = { [two] = three }
@@ -272,8 +344,9 @@ function fonts.afm.add_kerns(afmdata)
             if v.kerns then
                 local k = { }
                 for complex,simple in pairs(characters.uncomposed[what]) do
-                    if k[simple] and not k[complex] then
-                        k[complex] = k[simple]
+                    local ks = k[simple]
+                    if ks and not k[complex] then
+                        k[complex] = ks
                     end
                 end
                 if not table.is_empty(k) then
@@ -305,23 +378,30 @@ end
 --ldx]]--
 
 -- once we have otf sorted out (new format) we can try to make the afm
--- cache similar to it
+-- cache similar to it (similar tables)
+
+function fonts.afm.add_dimensions(data) -- we need to normalize afm to otf i.e. indexed table instead of name
+    if data then
+        for n, d in pairs(data.characters) do
+            local bb = d.boundingbox
+            if bb then
+                local ht, dp = bb[4], -bb[2]
+                if ht ~= 0 then d.height = ht end
+                if dp ~= 0 then d.depth  = dp end
+            end
+            d.name = n
+        end
+    end
+end
 
 function fonts.afm.copy_to_tfm(data)
     if data and data.characters then
         local tfm = { characters = { }, parameters = { } }
-        local characters = data.characters
-        if characters then
-            for k, v in pairs(characters) do
-                local t = { }
-                t.height      =   v.boundingbox[4]
-                t.depth       = - v.boundingbox[2]
-                t.width       =   v.wx
-                t.boundingbox =   v.boundingbox
-                t.index       =   v.index
-                t.name        =   k
-                t.unicode     =   v.unicode
-                tfm.characters[t.unicode] = t
+        local afmcharacters = data.characters
+        local characters, parameters = tfm.characters, tfm.parameters
+        if afmcharacters then
+            for k, v in pairs(afmcharacters) do
+                characters[v.unicode] = { description = v }
             end
         end
         tfm.encodingbytes      = data.encodingbytes or 2
@@ -340,26 +420,26 @@ function fonts.afm.copy_to_tfm(data)
         local spaceunits = 500
         tfm.spacer = "500 units"
         if data.isfixedpitch then
-            if characters['space'] and characters['space'].wx then
-                spaceunits, tfm.spacer = characters['space'].wx, "space"
-            elseif characters['emdash'] and characters['emdash'].wx then -- funny default
-                spaceunits, tfm.spacer = characters['emdash'].wx, "emdash"
+            if afmcharacters['space'] and afmcharacters['space'].width then
+                spaceunits, tfm.spacer = afmcharacters['space'].width, "space"
+            elseif afmcharacters['emdash'] and afmcharacters['emdash'].width then -- funny default
+                spaceunits, tfm.spacer = afmcharacters['emdash'].width, "emdash"
             elseif data.charwidth then
                 spaceunits, tfm.spacer = data.charwidth, "charwidth"
             end
-        elseif characters['space'] and characters['space'].wx then
-            spaceunits, tfm.spacer = characters['space'].wx, "space"
+        elseif afmcharacters['space'] and afmcharacters['space'].width then
+            spaceunits, tfm.spacer = afmcharacters['space'].width, "space"
         elseif data.charwidth then
             spaceunits, tfm.spacer = data.charwidth, "charwidth variable"
         end
         spaceunits = tonumber(spaceunits)
-        tfm.parameters[1] = 0          -- slant
-        tfm.parameters[2] = spaceunits -- space
-        tfm.parameters[3] = 500        -- space_stretch
-        tfm.parameters[4] = 333        -- space_shrink
-        tfm.parameters[5] = 400        -- x_height
-        tfm.parameters[6] = 1000       -- quad
-        tfm.parameters[7] = 0          -- extra_space (todo)
+        parameters.slant         = 0
+        parameters.space         = spaceunits
+        parameters.space_stretch = 500
+        parameters.space_shrink  = 333
+        parameters.x_height      = 400
+        parameters.quad          = 1000
+        parameters.extra_space   = 0
         if spaceunits < 200 then
             -- todo: warning
         end
@@ -367,28 +447,27 @@ function fonts.afm.copy_to_tfm(data)
         tfm.ascender    = math.abs(data.ascender  or 0)
         tfm.descender   = math.abs(data.descender or 0)
         if data.italicangle then
-            tfm.parameters[1] = tfm.parameters[1] - math.round(math.tan(data.italicangle*math.pi/180))
+            parameters.slant = parameters.slant - math.round(math.tan(data.italicangle*math.pi/180))
         end
         if data.isfixedpitch then
-          tfm.parameters[3] = 0
-          tfm.parameters[4] = 0
+            parameters.space_stretch = 0
+            parameters.space_shrink  = 0
         elseif fonts.afm.syncspace then
-            -- too little
-            -- tfm.parameters[3] = .2*spaceunits  -- space_stretch
-            -- tfm.parameters[4] = .1*spaceunits  -- space_shrink
-            -- taco's suggestion:
-            -- tfm.parameters[3] = .4*spaceunits  -- space_stretch
-            -- tfm.parameters[4] = .1*spaceunits  -- space_shrink
-            -- knuthian values: (for the moment compatible)
-            tfm.parameters[3] = spaceunits/2  -- space_stretch
-            tfm.parameters[4] = spaceunits/3  -- space_shrink
+            parameters.space_stretch = spaceunits/2
+            parameters.space_shrink  = spaceunits/3
         end
         if data.xheight and data.xheight > 0 then
-            tfm.parameters[5] = data.xheight
-        elseif tfm.characters['x'] and tfm.characters['x'].height then
-            tfm.parameters[5] = tfm.characters['x'].height
+            parameters.x_height = data.xheight
+        elseif afmcharacters['x'] and afmcharacters['x'].height then
+            parameters.x_height = afmcharacters['x'].height or 0
         end
-        if table.is_empty(tfm.characters) then
+        local fd = data.fontdimens
+        if fd and fd[8] and fd[9] and fd[10] then -- math
+            for k,v in pairs(fd) do
+                parameters[k] = v
+            end
+        end
+        if table.is_empty(characters) then
             return nil
         else
             return tfm
@@ -413,7 +492,7 @@ end
 function fonts.afm.set_features(tfmdata)
     local shared = tfmdata.shared
     local afmdata = shared.afmdata
-    shared.features = fonts.define.check(shared.features,fonts.afm.features.default)
+    -- elsewhere: shared.features = fonts.define.check(shared.features,fonts.afm.features.default)
     local features = shared.features
 --~ texio.write_nl(table.serialize(features))
     if not table.is_empty(features) then
@@ -437,6 +516,7 @@ function fonts.afm.set_features(tfmdata)
             end
             initialize(fonts.triggers)
             initialize(fonts.afm.features.list)
+            initialize(fonts.manipulators)
         end
         local fm = fonts.methods[mode]
         if fm and fm.afm then
@@ -458,21 +538,32 @@ function fonts.afm.set_features(tfmdata)
     end
 end
 
+function fonts.afm.check_features(specification)
+    local features, done = fonts.define.check(specification.features.normal,fonts.afm.features.default)
+    if done then
+        specification.features.normal = features
+        fonts.tfm.hash_instance(specification,true)
+    end
+end
+
 function fonts.afm.afm_to_tfm(specification)
     local afmname = specification.filename or specification.name
     local encoding, filename = afmname:match("^(.-)%-(.*)$") -- context: encoding-name.*
     if encoding and filename and fonts.enc.known[encoding] then
--- only when no bla-name is found
         fonts.tfm.set_normal_feature(specification,'encoding',encoding) -- will go away
         if fonts.trace then
-            logs.report("define font", string.format("stripping encoding prefix from filename %s",afmname))
+            logs.report("load afm", string.format("stripping encoding prefix from filename %s",afmname))
         end
         afmname = filename
+    elseif specification.forced == "afm" then
+        if fonts.trace then
+            logs.report("load afm", string.format("forcing afm format for %s",afmname))
+        end
     else
         local tfmname = input.findbinfile(texmf.instance,afmname,"ofm") or ""
         if tfmname ~= "" then
             if fonts.trace then
-                logs.report("define font", string.format("fallback from afm to tfm for %s",afmname))
+                logs.report("load afm", string.format("fallback from afm to tfm for %s",afmname))
             end
             afmname = ""
         end
@@ -480,12 +571,15 @@ function fonts.afm.afm_to_tfm(specification)
     if afmname == "" then
         return nil
     else
+        fonts.afm.check_features(specification)
+        specification = fonts.define.resolve(specification) -- new, was forgotten
         local features = specification.features.normal
         local cache_id = specification.hash
-        local tfmdata  = containers.read(fonts.tfm.cache, cache_id) -- cache with features applied
+        local tfmdata  = containers.read(fonts.tfm.cache(), cache_id) -- cache with features applied
         if not tfmdata then
             local afmdata = fonts.afm.load(afmname)
             if not table.is_empty(afmdata) then
+                fonts.afm.add_dimensions(afmdata)
                 tfmdata = fonts.afm.copy_to_tfm(afmdata)
                 if not table.is_empty(tfmdata) then
                     tfmdata.shared = tfmdata.shared or { }
@@ -495,9 +589,9 @@ function fonts.afm.afm_to_tfm(specification)
                     fonts.afm.set_features(tfmdata)
                 end
             elseif fonts.trace then
-                logs.report("define font", string.format("no (valid) afm file found with name %s",afmname))
+                logs.report("load afm", string.format("no (valid) afm file found with name %s",afmname))
             end
-            tfmdata = containers.write(fonts.tfm.cache,cache_id,tfmdata)
+            tfmdata = containers.write(fonts.tfm.cache(),cache_id,tfmdata)
         end
         return tfmdata
     end
@@ -522,6 +616,10 @@ function fonts.tfm.set_normal_feature(specification,name,value)
 end
 
 function fonts.tfm.read_from_afm(specification)
+--~ local fullname = input.findbinfile(texmf.instance,specification.name,"afm") or ""
+--~ if fullname ~= "" then
+--~     specification.filename = fullname
+--~ end
     local tfmtable = fonts.afm.afm_to_tfm(specification)
     if tfmtable then
         tfmtable.name = specification.name
@@ -561,8 +659,8 @@ function fonts.afm.features.prepare_ligatures(tfmdata,ligatures,value) -- probab
                 if al then
                     local ligatures = { }
                     for k,v in pairs(al) do
-                        ligatures[charlist[k].index] = {
-                            char = charlist[v].index,
+                        ligatures[charlist[k].unicode] = {
+                            char = charlist[v].unicode,
                             type = 0
                         }
                     end
@@ -577,11 +675,11 @@ function fonts.afm.features.prepare_kerns(tfmdata,kerns,value)
     if value then
         local charlist = tfmdata.shared.afmdata.characters
         for _, chr in pairs(tfmdata.characters) do
-            local newkerns = charlist[chr.name][kerns]
+            local newkerns = charlist[chr.description.name][kerns]
             if newkerns then
                 local t = chr.kerns or { }
                 for k,v in pairs(newkerns) do
-                    t[charlist[k].index] = v
+                    t[charlist[k].unicode] = v
                 end
                 chr.kerns = t
             end
@@ -589,25 +687,16 @@ function fonts.afm.features.prepare_kerns(tfmdata,kerns,value)
     end
 end
 
-function fonts.initializers.base.afm.ligatures(tfmdata,value)
-    fonts.afm.features.prepare_ligatures(tfmdata,'ligatures',value)
-end
+-- hm, register?
 
-function fonts.initializers.base.afm.texligatures(tfmdata,value)
-    fonts.afm.features.prepare_ligatures(tfmdata,'texligatures',value)
-end
-
-function fonts.initializers.base.afm.kerns(tfmdata,value)
-    fonts.afm.features.prepare_kerns(tfmdata,'kerns',value)
-end
-
-function fonts.initializers.base.afm.extrakerns(tfmdata,value)
-    fonts.afm.features.prepare_kerns(tfmdata,'extrakerns',value)
-end
+function fonts.initializers.base.afm.ligatures   (tfmdata,value) fonts.afm.features.prepare_ligatures(tfmdata,'ligatures',   value) end
+function fonts.initializers.base.afm.texligatures(tfmdata,value) fonts.afm.features.prepare_ligatures(tfmdata,'texligatures',value) end
+function fonts.initializers.base.afm.kerns       (tfmdata,value) fonts.afm.features.prepare_kerns    (tfmdata,'kerns',       value) end
+function fonts.initializers.base.afm.extrakerns  (tfmdata,value) fonts.afm.features.prepare_kerns    (tfmdata,'extrakerns',  value) end
 
 fonts.afm.features.register('liga',true)
 fonts.afm.features.register('kerns',true)
-fonts.afm.features.register('extrakerns')
+fonts.afm.features.register('extrakerns') -- needed?
 
 fonts.initializers.node.afm.ligatures    = fonts.initializers.base.afm.ligatures
 fonts.initializers.node.afm.texligatures = fonts.initializers.base.afm.texligatures
@@ -619,6 +708,12 @@ fonts.initializers.node.afm.liga         = fonts.initializers.base.afm.ligatures
 fonts.initializers.base.afm.tlig         = fonts.initializers.base.afm.texligatures
 fonts.initializers.node.afm.tlig         = fonts.initializers.base.afm.texligatures
 
+fonts.initializers.base.afm.trep         = fonts.tfm.replacements
+fonts.initializers.node.afm.trep         = fonts.tfm.replacements
+
+fonts.afm.features.register('tlig',true) -- todo: also proper features for afm
+fonts.afm.features.register('trep',true) -- todo: also proper features for afm
+
 -- tfm features
 
 fonts.initializers.base.afm.equaldigits = fonts.initializers.common.equaldigits
@@ -626,6 +721,11 @@ fonts.initializers.node.afm.equaldigits = fonts.initializers.common.equaldigits
 fonts.initializers.base.afm.lineheight  = fonts.initializers.common.lineheight
 fonts.initializers.node.afm.lineheight  = fonts.initializers.common.lineheight
 
+-- vf features
+
+fonts.initializers.base.afm.compose = fonts.initializers.common.compose
+fonts.initializers.node.afm.compose = fonts.initializers.common.compose
+
 -- afm specific, encodings ...kind of obsolete
 
 fonts.afm.features.register('encoding')
@@ -633,4 +733,13 @@ fonts.afm.features.register('encoding')
 fonts.initializers.base.afm.encoding = fonts.initializers.common.encoding
 fonts.initializers.node.afm.encoding = fonts.initializers.common.encoding
 
--- todo: oldstyle smallcaps as features for afm files
+-- todo: oldstyle smallcaps as features for afm files (use with care)
+
+fonts.initializers.base.afm.onum  = fonts.initializers.common.oldstyle
+fonts.initializers.base.afm.smcp  = fonts.initializers.common.smallcaps
+fonts.initializers.base.afm.fkcp  = fonts.initializers.common.fakecaps
+
+fonts.afm.features.register('onum',false)
+fonts.afm.features.register('smcp',false)
+fonts.afm.features.register('fkcp',false)
+
