@@ -15,8 +15,9 @@ if not modules then modules = { } end modules ['font-ini'] = {
 
 fonts = fonts or { }
 
-fonts.trace = false -- true
-fonts.mode  = 'base'
+fonts.trace   = false -- true
+fonts.mode    = 'base'
+fonts.private = 0xE000
 
 fonts.methods = {
     base = { tfm = { }, afm = { }, otf = { }, vtf = { }, fix = { } },
@@ -31,9 +32,17 @@ fonts.initializers = {
 fonts.triggers = {
     'mode',
     'language',
-    'script'
+    'script',
+    'strategy',
 }
 
+fonts.manipulators = {
+}
+
+fonts.define                  = fonts.define                  or { }
+fonts.define.specify          = fonts.define.specify          or { }
+fonts.define.specify.synonyms = fonts.define.specify.synonyms or { }
+
 -- tracing
 
 do
@@ -42,7 +51,7 @@ do
 
     fonts.color.trace = false
 
-    local attribute = attributes.numbers['color'] or 4 -- we happen to know this -)
+    local attribute = attributes.numbers['color'] or 7 -- we happen to know this -)
     local mapping   = attributes.list[attribute]
 
     local set_attribute   = node.set_attribute
