@@ -86,6 +86,7 @@ function fonts.tfm.read_from_tfm(specification)
                         tfmdata.fonts = vfdata.fonts
                     end
                 end
+--~ print(table.serialize(tfmdata))
             end
             fonts.tfm.enhance(tfmdata,specification)
         end
@@ -103,22 +104,45 @@ do with the fact that <l n='tex'/> uses a negative multiple of 1000 as
 a signal for a font scaled based on the design size.</p>
 --ldx]]--
 
+do
+
+    local factors = {
+        pt = 65536.0,
+        bp = 65781.8,
+    }
+
+    function fonts.tfm.setfactor(f)
+        fonts.tfm.factor = factors[f or 'pt'] or factors.pt
+    end
+
+    fonts.tfm.setfactor()
+
+end
+
 function fonts.tfm.scaled(scaledpoints, designsize) -- handles designsize in sp as well
     if scaledpoints < 0 then
         if designsize then
-            if designsize > 65536 then -- or just 1000
+            if designsize > fonts.tfm.factor then -- or just 1000 / when? mp?
                 return (- scaledpoints/1000) * designsize -- sp's
             else
-                return (- scaledpoints/1000) * designsize * 65536
+                return (- scaledpoints/1000) * designsize * fonts.tfm.factor
             end
         else
-            return (- scaledpoints/1000) * 10 * 65536
+            return (- scaledpoints/1000) * 10 * fonts.tfm.factor
         end
     else
         return scaledpoints
     end
 end
 
+--~ function fonts.tfm.scaled(scaledpoints, designsize)
+--~     if scaledpoints < 0 then
+--~         return (- scaledpoints/1000) * (designsize or 10) * fonts.tfm.factor
+--~     else
+--~         return scaledpoints
+--~     end
+--~ end
+
 --[[ldx--
 <p>Before a font is passed to <l n='tex'/> we scale it. Here we also need
 to scale virtual characters.</p>
@@ -137,7 +161,7 @@ function fonts.tfm.get_virtual_id(tfmdata)
 end
 
 function fonts.tfm.check_virtual_id(tfmdata, id)
-    if tfmdata.type == "virtual" then
+    if tfmdata and tfmdata.type == "virtual" then
         if not tfmdata.fonts or #tfmdata.fonts == 0 then
             tfmdata.type, tfmdata.fonts = "real", nil
         else
@@ -150,66 +174,106 @@ function fonts.tfm.check_virtual_id(tfmdata, id)
     end
 end
 
--- if t.tounicode = 1 then also characters[n].tounicode = "string"
+--[[ldx--
+<p>Beware, the boundingbox is passed as reference so we may not overwrite it
+in the process; numbers are of course copies. Here 65536 equals 1pt. (Due to
+excessive memory usage in CJK fonts, we no longer pass the boundingbox.)</p>
+--ldx]]--
 
-function fonts.tfm.scale(tfmtable, scaledpoints)
-    -- 65536 = 1pt
-    -- 1000 units per designsize (not always)
-    local scale, round = tex.scale, tex.round -- replaces math.floor(n*m+0.5)
-    local delta
+fonts.trace_scaling = false
+
+function fonts.tfm.do_scale(tfmtable, scaledpoints)
+    local trace = fonts.trace_scaling
     if scaledpoints < 0 then
         scaledpoints = (- scaledpoints/1000) * tfmtable.designsize -- already in sp
     end
-    delta = scaledpoints/(tfmtable.units or 1000) -- brr, some open type fonts have 2048
+    local delta = scaledpoints/(tfmtable.units or 1000) -- brr, some open type fonts have 2048
     local t = { }
     t.factor = delta
     for k,v in pairs(tfmtable) do
-        if type(v) == "table" then
-            t[k] = { }
-        else
-            t[k] = v
-        end
+        t[k] = (type(v) == "table" and { }) or v
     end
+    -- new
+    if tfmtable.fonts then
+        t.fonts = table.fastcopy(tfmtable.fonts)
+    end
+    -- local zerobox = { 0, 0, 0, 0 }
+    local tp = t.parameters
+    local tfmp = tfmtable.parameters -- let's check for indexes
+    tp.slant         = (tfmp.slant         or tfmp[1] or 0)
+    tp.space         = (tfmp.space         or tfmp[2] or 0)*delta
+    tp.space_stretch = (tfmp.space_stretch or tfmp[3] or 0)*delta
+    tp.space_shrink  = (tfmp.space_shrink  or tfmp[4] or 0)*delta
+    tp.x_height      = (tfmp.x_height      or tfmp[5] or 0)*delta
+    tp.quad          = (tfmp.quad          or tfmp[6] or 0)*delta
+    tp.extra_space   = (tfmp.extra_space   or tfmp[7] or 0)*delta
+    local protrusionfactor = (tp.quad ~= 0 and 1000/tp.quad) or 0
     local tc = t.characters
     for k,v in pairs(tfmtable.characters) do
+        local description = v.description or v -- shared data
         local chr = {
-            unicode = v.unicode,
-            name    = v.name,
-            index   = v.index or k,
-            width   = scale(v.width , delta),
-            height  = scale(v.height, delta),
-            depth   = scale(v.depth , delta),
-            class   = v.class
+            description = description,
+            unicode = description.unicode,
+            name    = description.name,
+            index   = description.index or k,
+            width   = delta*(description.width  or 0),
+            height  = delta*(description.height or 0),
+            depth   = delta*(description.depth  or 0),
+            class   = description.class
         }
-if fonts.trace then
-    logs.report("define font", string.format("n=%s, u=%s, i=%s, n=%s c=%s",k,v.unicode,v.index,v.name or '-',v.class or '-'))
-end
-        local b = v.boundingbox -- maybe faster to have llx etc not in table
-        if b then
-            chr.boundingbox = scale(v.boundingbox,delta)
+        if trace then
+            logs.report("define font", string.format("n=%s, u=%s, i=%s, n=%s c=%s",k,description.unicode,description.index,description.name or '-',description.class or '-'))
         end
-        if v.italic then
-            chr.italic = scale(v.italic,delta)
+    --  local vb = v.boundingbox
+    --  if vb then
+    --      chr.boundingbox = { vb[1]*delta, vb[2]*delta, vb[3]*delta, vb[4]*delta }
+    --  else
+    --  --  chr.boundingbox = zerobox -- most afm en otf files have bboxes so ..
+    --  end
+        local ve = v.expansion_factor
+        if ve then
+            chr.expansion_factor = ve*1000 -- expansionfactor
         end
-        if v.kerns then
-            chr.kerns = scale(v.kerns,delta)
+        local vl = v.left_protruding
+        if vl then
+            chr.left_protruding  = protrusionfactor*chr.width*vl
         end
-        if v.ligatures then
-            local tt = { }
-            for kk,vv in pairs(v.ligatures) do
-                tt[kk] = vv
+        local vr = v.right_protruding
+        if vr then
+            chr.right_protruding  = protrusionfactor*chr.width*vr
+        end
+        local vi = description.italic
+        if vi then
+            chr.italic = vi*delta
+        end
+        local vk = v.kerns
+        if vk then
+            local tt = {}
+            for k,v in pairs(vk) do tt[k] = v*delta end
+            chr.kerns = tt
+        end
+        local vl = v.ligatures
+        if vl then
+            if true then
+                chr.ligatures = vl -- shared
+            else
+                local tt = { }
+                for i,l in pairs(vl) do
+                    tt[i] = l
+                end
+                chr.ligatures = tt
             end
-            chr.ligatures = tt
         end
-        if v.commands then
+        local vc = v.commands
+        if vc then
             -- we assume non scaled commands here
-            local vc, tt = v.commands, { }
+            local tt = { }
             for i=1,#vc do
                 local ivc = vc[i]
                 local key = ivc[1]
-                if key == "right" or key == "left" then
-                    tt[#tt+1] = { key, scale(ivc[2],delta) }
-                else
+                if key == "right" or key == "left" or key == "down" or key == "up" then
+                    tt[#tt+1] = { key, ivc[2]*delta }
+                else -- not comment
                     tt[#tt+1] = ivc -- shared since in cache and untouched
                 end
             end
@@ -217,28 +281,29 @@ end
         end
         tc[k] = chr
     end
-    local tp = t.parameters
-    for k,v in pairs(tfmtable.parameters) do
-        if k == 1 then
-            tp[k] = round(v)
-        else
-            tp[k] = scale(v,delta)
-        end
-    end
     -- t.encodingbytes, t.filename, t.fullname, t.name: elsewhere
-    t.size        = scaledpoints
-    t.italicangle = tfmtable.italicangle
-    t.ascender    = scale(tfmtable.ascender  or 0,delta)
-    t.descender   = scale(tfmtable.descender or 0,delta)
-    t.shared      = tfmtable.shared or { }
-    if t.unique then
-        t.unique = table.fastcopy(tfmtable.unique)
-    else
-        t.unique = { }
-    end
+    t.size = scaledpoints
     if t.fonts then
         t.fonts = table.fastcopy(t.fonts) -- maybe we virtualize more afterwards
     end
+    return t, delta
+end
+
+--[[ldx--
+<p>The reason why the scaler is split, is that for a while we experimented
+with a helper function. However, in practice the <l n='api'/> calls are too slow to
+make this profitable and the <l n='lua'/> based variant was just faster. A days
+wasted day but an experience richer.</p>
+--ldx]]--
+
+function fonts.tfm.scale(tfmtable, scaledpoints)
+    local t, factor = fonts.tfm.do_scale(tfmtable, scaledpoints)
+    t.factor    = factor
+    t.ascender  = factor*(tfmtable.ascender  or 0)
+    t.descender = factor*(tfmtable.descender or 0)
+    t.shared    = tfmtable.shared or { }
+    t.unique    = table.fastcopy(tfmtable.unique or {})
+--~ print("scaling", t.name, t.factor) -- , fonts.tfm.hash_features(tfmtable.specification))
     return t
 end
 
@@ -261,11 +326,16 @@ function fonts.logger.save(tfmtable,source,specification) -- save file name in s
 end
 
 function fonts.logger.report(separator)
-    local t = { }
-    for _,v in pairs(table.sortedkeys(fonts.loaded)) do
-        t[#t+1] = v .. ":" .. fonts.loaded[v].source
+    local s = table.sortedkeys(fonts.loaded)
+    if #s > 0 then
+        local t = { }
+        for _,v in ipairs(s) do
+            t[#t+1] = v .. ":" .. fonts.loaded[v].source
+        end
+        return table.concat(t,separator or " ")
+    else
+        return "none"
     end
-    return table.concat(t,separator or " ")
 end
 
 function fonts.logger.format(name)
@@ -342,86 +412,6 @@ function fonts.initializers.common.lineheight(tfmdata,value)
     end
 end
 
-
---[[ldx--
-<p>The following feature is kind of experimental and deals with fallback characters.</p>
---ldx]]--
-
-fonts.initializers.complements      = fonts.initializers.complements      or { }
-fonts.initializers.complements.data = fonts.initializers.complements.data or { }
-
-function fonts.initializers.complements.load(pattern)
-    local data = fonts.initializers.complements.data[pattern]
-    if not data then
-        data = { }
-        for k,v in pairs(characters.data) do
-            local vd = v.description
-            if vd and vd:find(pattern) then
-                local vs = v.specials
-                if vs and vs[1] == "compat" then
-                    data[#data+1] = k
-                end
-            end
-        end
-        fonts.initializers.complements.data[pattern] = data
-    end
-    return data
-end
-
-function fonts.initializers.common.complement(tfmdata,value) -- todo: value = latin:compat,....
-    if value then
-        local chr, index, data, get_virtual_id = tfmdata.characters, nil, characters.data, fonts.tfm.get_virtual_id
-        local selection = fonts.initializers.complements.load("LATIN") -- will be value
-        for _, k in ipairs(selection) do
-            if not chr[k] then
-                local dk = data[k]
-                local vs, name = dk.specials, dk.adobename
-                index = index or get_virtual_id(tfmdata)
-                local ok, t, w, h, d, krn, pre = true, {}, 0, 0, 0, nil, nil
-                for i=2,#vs do
-                    local vsi = vs[i]
-                    local c = chr[vsi]
-                    if c then
-                        local k = krn and krn[vsi]
-                        if k then
-                            t[#t+1] = { 'right', k }
-                            w = w + k
-                        end
-                        t[#t+1] = { 'slot', index, vsi }
-                        w = w + c.width
-                        h = h + c.height
-                        d = d + c.depth
-                        krn = c.kerns
-                    else
-                        ok = false
-                        break
-                    end
-                end
-                if ok then
-                    chr[k] = {
-                        unicode  = k,
-                        name     = name,
-                        commands = t,
-                        width    = w,
-                        height   = h,
-                        depth    = d,
-                        kerns    = krn
-                    }
-                    local c = vs[2]
-                    for k,v in pairs(chr) do -- slow
-                        local krn = v.kerns
-                        if krn then
-                            krn[k] = krn[c]
-                        end
-                    end
-                end
-            end
-        end
-    end
-end
-
-table.insert(fonts.triggers,"complement")
-
 --[[ldx--
 <p>It does not make sense any more to support messed up encoding vectors
 so we stick to those that implement oldstyle and small caps. After all,
@@ -448,7 +438,7 @@ do
                 local characters = tfmdata.characters
                 local unicodes = afmdata.luatex.unicodes
                 local function remap(pattern,name)
-                    local p = lpeg.match(pattern,name)
+                    local p = pattern:match(name)
                     if p then
                         local oldchr, newchr = unicodes[p], unicodes[name]
                         if oldchr and newchr then
@@ -475,18 +465,18 @@ do
             local afmdata = tfmdata.shared.afmdata
             local characters = tfmdata.characters
             local unicodes = afmdata.luatex.unicodes
-            local function remap(pattern,name)
-                local p = lpeg.match(pattern,name)
-                if p then
-                    local oldchr, newchr = unicodes[p], unicodes[name]
-                    if oldchr and newchr then
-                        characters[oldchr] = characters[newchr]
+            local done = false
+            for i, blob in pairs(characters) do
+                local name = blob.description.name
+                if name then
+                    local p = pattern:match(name)
+                    if p then
+                        local oldchr, newchr = unicodes[p], unicodes[name]
+                        if oldchr and newchr then
+                            characters[oldchr] = characters[newchr]
+                        end
                     end
                 end
-                return p
-            end
-            for _, blob in pairs(characters) do
-                remap(pattern,blob.name)
             end
         end
     end
@@ -498,6 +488,27 @@ do
         fonts.initializers.common.remap(tfmdata,value,smallcaps)
     end
 
+    function fonts.initializers.common.fakecaps(tfmdata,value)
+        if value then
+            -- todo: scale down
+            local afmdata = tfmdata.shared.afmdata
+            local characters = tfmdata.characters
+            local unicodes = afmdata.luatex.unicodes
+            for i, blob in pairs(characters) do
+                local name = blob.description.name
+                if name then
+                    local p = name:lower()
+                    if p then
+                        local oldchr, newchr = unicodes[p], unicodes[name]
+                        if oldchr and newchr then
+                            characters[oldchr] = characters[newchr]
+                        end
+                    end
+                end
+            end
+        end
+    end
+
 end
 
 --~ function fonts.initializers.common.install(format,feature) -- 'afm','lineheight'
@@ -520,8 +531,8 @@ do
     local glyph           = node.id('glyph')
     local fontdata        = fonts.tfm.id
     local set_attribute   = node.set_attribute
-    local unset_attribute = node.unset_attribute
-    local has_attribute   = node.has_attribute
+--  local unset_attribute = node.unset_attribute
+--  local has_attribute   = node.has_attribute
 
     local state = attributes.numbers['state'] or 100
 
@@ -533,32 +544,43 @@ do
     function fonts.analyzers.aux.setstate(head,font)
         local characters = fontdata[font].characters
         local first, last, current, n, done = nil, nil, head, 0, false -- maybe make n boolean
-        local function finish()
-            if first and first == last then
-                set_attribute(last,state,4) -- isol
-            elseif last then
-                set_attribute(last,state,3) -- fina
-            end
-            first, last, n = nil, nil, 0
-        end
         while current do
             if current.id == glyph and current.font == font then
-                if characters[current.char].class == "mark" then
-                    done = true
-                    set_attribute(current,state,5) -- mark
-                elseif n == 0 then
-                    first, last, n = current, current, 1
-                    set_attribute(current,state,1) -- init
-                else
-                    last, n = current, n+1
-                    set_attribute(current,state,2) -- medi
+                local c = characters[current.char]
+                if c then
+                    if c.description.class == "mark" then
+                        done = true
+                        set_attribute(current,state,5) -- mark
+                    elseif n == 0 then
+                        first, last, n = current, current, 1
+                        set_attribute(current,state,1) -- init
+                    else
+                        last, n = current, n+1
+                        set_attribute(current,state,2) -- medi
+                    end
+                else -- finish
+                    if first and first == last then
+                        set_attribute(last,state,4) -- isol
+                    elseif last then
+                        set_attribute(last,state,3) -- fina
+                    end
+                    first, last, n = nil, nil, 0
                 end
-            else
-                finish()
+            else -- finish
+                if first and first == last then
+                    set_attribute(last,state,4) -- isol
+                elseif last then
+                    set_attribute(last,state,3) -- fina
+                end
+                first, last, n = nil, nil, 0
             end
             current = current.next
         end
-        finish()
+        if first and first == last then
+            set_attribute(last,state,4) -- isol
+        elseif last then
+            set_attribute(last,state,3) -- fina
+        end
         return head, done
     end
 
@@ -581,11 +603,12 @@ do
         local current, last, done, n = head, nil, false, 0
         while current do
             if current.id == glyph and current.font == font then
-                if characters[current.char].class == "mark" then
+                local c = characters[current.char]
+                if c and c.description.class == "mark" then
                     -- check if head
                     if last and not last.components then
                         last.components = current
-                        last.components.prev = nil
+                        current.prev = nil -- last.components.prev = nil
                         done = true
                         n = 1
                     else
@@ -624,7 +647,7 @@ do
     function fonts.removemarks(head,font)
         local current, done, characters = head, false, tfmdata.characters
         while current do
-            if current.id == glyph and current.font == font and characters[current.char].class == "mark" then
+            if current.id == glyph and current.font == font and characters[current.char].description.class == "mark" then
                 local next, prev = current.next, current.prev
                 if next then
                     next.prev = prev
@@ -677,3 +700,22 @@ do
     end
 
 end
+
+function fonts.tfm.replacements(tfm,value)
+--~     tfm.characters[0x0022] = table.fastcopy(tfm.characters[0x201D])
+--~     tfm.characters[0x0027] = table.fastcopy(tfm.characters[0x2019])
+--~     tfm.characters[0x0060] = table.fastcopy(tfm.characters[0x2018])
+--~     tfm.characters[0x0022] = tfm.characters[0x201D]
+    tfm.characters[0x0027] = tfm.characters[0x2019]
+--~     tfm.characters[0x0060] = tfm.characters[0x2018]
+end
+
+-- auto complete font with missing composed characters
+
+table.insert(fonts.manipulators,"compose")
+
+function fonts.initializers.common.compose(tfmdata,value)
+    if value then
+        fonts.vf.aux.compose_characters(tfmdata)
+    end
+end
