@@ -67,14 +67,24 @@ if texconfig and not texlua then
                     input.logger('= ' .. tag .. ' closer (' .. unicode.utfname[u] .. ')',filename)
                     input.show_close(filename)
                 end,
+--~                 getline = function(n)
+--~                     local line = t.lines[n]
+--~                     if not line or line == "" then
+--~                         return ""
+--~                     else
+--~                         local translator = input.filters.utf_translator
+--~                         return (translator and translator(line)) or line
+--~                     end
+--~                 end,
                 reader = function(self)
                     self = self or t
                     local current, lines = self.current, self.lines
                     if current >= #lines then
                         return nil
                     else
-                        self.current = current + 1
-                        local line = lines[self.current]
+                        current = current + 1
+                        self.current = current
+                        local line = lines[current]
                         if line == "" then
                             return ""
                         else
@@ -92,18 +102,22 @@ if texconfig and not texlua then
         else
             input.logger('+ ' .. tag .. ' opener',filename)
             -- todo: file;name -> freeze / eerste regel scannen -> freeze
+            local filters = input.filters
             t = {
                 reader = function(self)
                     local line = file_handle:read()
                     if line == "" then
                         return ""
-                    elseif input.filters.utf_translator then
-                        return input.filters.utf_translator(line)
-                    elseif input.filters.dynamic_translator then
-                        return input.filters.dynamic_translator(line)
-                    else
-                        return line
                     end
+                    local translator = filters.utf_translator
+                    if translator then
+                        return translator(line)
+                    end
+                    translator = filters.dynamic_translator
+                    if translator then
+                        return translator(line)
+                    end
+                    return line
                 end,
                 close = function()
                     input.logger('= ' .. tag .. ' closer',filename)
@@ -165,6 +179,104 @@ end
 
 -- callback into the file io and related things; disabling kpse
 
+
+if texconfig and not texlua then do
+
+    -- this is not the right place, because we refer to quite some not yet defined tables, but who cares ...
+
+    ctx = ctx or { }
+
+    local ss = { }
+
+    function ctx.writestatus(a,b)
+        local s = ss[a]
+        if not ss[a] then
+            s = a:rpadd(15) .. ": "
+            ss[a] = s
+        end
+        texio.write_nl(s .. b .. "\n")
+    end
+
+    -- this will become: ctx.install_statistics(fnc() return ..,.. end) etc
+
+    local statusinfo, n = { }, 0
+
+    function ctx.register_statistics(tag,pattern,fnc)
+        statusinfo[#statusinfo+1] = { tag, pattern, fnc }
+        if #tag > n then n = #tag end
+    end
+
+    function ctx.show_statistics() -- todo: move calls
+        if caches then
+            ctx.register_statistics("used config path", "%s", function() return caches.configpath(texmf.instance) end)
+            ctx.register_statistics("used cache path", "%s", function() return caches.path end)
+        end
+        if status.luabytecodes > 0 and input.storage and input.storage.done then
+            ctx.register_statistics("modules/dumps/instances", "%s/%s/%s", function() return status.luabytecodes-500, input.storage.done, status.luastates end)
+        end
+        if texmf.instance then
+            ctx.register_statistics("input load time", "%s seconds", function() return input.loadtime(texmf.instance) end)
+        end
+        if fonts then
+            ctx.register_statistics("fonts load time","%s seconds", function() return input.loadtime(fonts) end)
+        end
+        if xml then
+            ctx.register_statistics("xml load time", "%s seconds, backreferences: %i, outer filtering time: %s", function() return input.loadtime(xml), #lxml.self, input.loadtime(lxml) end)
+        end
+        if mptopdf then
+            ctx.register_statistics("mps conversion time", "%s seconds", function() return input.loadtime(mptopdf) end)
+        end
+        if nodes then
+            ctx.register_statistics("node processing time", "%s seconds (including kernel)", function() return input.loadtime(nodes) end)
+        end
+        if kernel then
+            ctx.register_statistics("kernel processing time", "%s seconds", function() return input.loadtime(kernel) end)
+        end
+        if attributes then
+            ctx.register_statistics("attribute processing time", "%s seconds", function() return input.loadtime(attributes) end)
+        end
+        if languages then
+            ctx.register_statistics("language load time", "%s seconds, n=%s", function() return input.loadtime(languages), languages.hyphenation.n() end)
+        end
+        if figures then
+            ctx.register_statistics("graphics processing time", "%s seconds, n=%s (including tex)", function() return input.loadtime(figures), figures.n or "?" end)
+        end
+        if metapost then
+            ctx.register_statistics("metapost processing time", "%s seconds, loading: %s seconds, execution: %s seconds, n: %s", function() return input.loadtime(metapost), input.loadtime(mplib), input.loadtime(metapost.exectime), metapost.n end)
+        end
+        if status.luastate_bytes then
+            ctx.register_statistics("current memory usage", "%s bytes", function() return status.luastate_bytes end)
+        end
+        if nodes then
+            ctx.register_statistics("cleaned up reserved nodes", "%s nodes, %s lists of %s", function() return nodes.cleanup_reserved(tex.count[24]) end) -- \topofboxstack
+        end
+        if status.node_mem_usage then
+            ctx.register_statistics("node memory usage", "%s", function() return status.node_mem_usage end)
+        end
+        if languages then
+            ctx.register_statistics("loaded patterns", "%s", function() return languages.logger.report() end)
+        end
+        if fonts then
+            ctx.register_statistics("loaded fonts", "%s", function() return fonts.logger.report() end)
+        end
+        if xml then -- so we are in mkiv, we need a different check
+            ctx.register_statistics("runtime", "%s seconds, %i processed pages, %i shipped pages, %.3f pages/second", function()
+                input.stoptiming(texmf)
+                local runtime = input.loadtime(texmf)
+                local shipped = tex.count['nofshipouts']
+                local pages = tex.count['realpageno'] - 1
+                local persecond = shipped / runtime
+                return runtime, pages, shipped, persecond
+            end)
+        end
+        for _, t in ipairs(statusinfo) do
+            local tag, pattern, fnc = t[1], t[2], t[3]
+            ctx.writestatus("mkiv lua stats", string.format("%s - %s", tag:rpadd(n," "), pattern:format(fnc())))
+        end
+    end
+
+end end
+
 if texconfig and not texlua then
 
     texconfig.kpse_init        = false
@@ -175,6 +287,8 @@ if texconfig and not texlua then
 
     if not texmf then texmf = { } end
 
+    input.starttiming(texmf)
+
     if not texmf.instance then
 
         if not texmf.instance then -- prevent a second loading
@@ -254,59 +368,61 @@ if texconfig and not texlua then
             function input.register_start_actions(f) table.insert(input.start_actions, f) end
             function input.register_stop_actions (f) table.insert(input.stop_actions,  f) end
 
---~             callback.register('start_run', function() for _, a in pairs(input.start_actions) do a() end end)
---~             callback.register('stop_run' , function() for _, a in pairs(input.stop_actions ) do a() end end)
+        --~ callback.register('start_run', function() for _, a in pairs(input.start_actions) do a() end end)
+        --~ callback.register('stop_run' , function() for _, a in pairs(input.stop_actions ) do a() end end)
 
         end
 
-        if callback and (input.logmode() == 'xml') then
+        if callback then
 
-            function input.start_page_number()
-                texio.write_nl("<p real='" .. tex.count[0] .. "' page='"..tex.count[1].."' sub='"..tex.count[2].."'")
-            end
-            function input.stop_page_number()
-                texio.write("/>")
-                texio.write_nl("")
-            end
+            if input.logmode() == 'xml' then
 
-            callback.register('start_page_number'  , input.start_page_number)
-            callback.register('stop_page_number'   , input.stop_page_number )
-
-            function input.report_output_pages(p,b)
-                texio.write_nl("<v k='pages'>"..p.."</v>")
-                texio.write_nl("<v k='bytes'>"..b.."</v>")
-                texio.write_nl("")
-            end
-            function input.report_output_log()
-            end
-
-            callback.register('report_output_pages', input.report_output_pages)
-            callback.register('report_output_log'  , input.report_output_log  )
-
-            function input.start_run()
-                texio.write_nl("<?xml version='1.0' standalone='yes'?>")
-                texio.write_nl("<job xmlns='www.tug.org/luatex/schemas/context-job.rng'>")
-                texio.write_nl("")
-            end
-            function input.stop_run()
-                texio.write_nl("</job>")
-            end
-            function input.show_statistics()
-                for k,v in pairs(status.list()) do
-                    texio.write_nl("log","<v k='"..k.."'>"..tostring(v).."</v>")
+                function input.start_page_number()
+                    texio.write_nl("<p real='" .. tex.count[0] .. "' page='"..tex.count[1].."' sub='"..tex.count[2].."'")
                 end
+                function input.stop_page_number()
+                    texio.write("/>")
+                    texio.write_nl("")
+                end
+
+                callback.register('start_page_number'  , input.start_page_number)
+                callback.register('stop_page_number'   , input.stop_page_number )
+
+                function input.report_output_pages(p,b)
+                    texio.write_nl("<v k='pages'>"..p.."</v>")
+                    texio.write_nl("<v k='bytes'>"..b.."</v>")
+                    texio.write_nl("")
+                end
+                function input.report_output_log()
+                end
+
+                callback.register('report_output_pages', input.report_output_pages)
+                callback.register('report_output_log'  , input.report_output_log  )
+
+                function input.start_run()
+                    texio.write_nl("<?xml version='1.0' standalone='yes'?>")
+                    texio.write_nl("<job xmlns='www.tug.org/luatex/schemas/context-job.rng'>")
+                    texio.write_nl("")
+                end
+                function input.stop_run()
+                    texio.write_nl("</job>")
+                end
+                function input.show_statistics()
+                    for k,v in pairs(status.list()) do
+                        texio.write_nl("log","<v k='"..k.."'>"..tostring(v).."</v>")
+                    end
+                end
+
+                table.insert(input.start_actions, input.start_run)
+                table.insert(input.stop_actions , input.show_statistics)
+                table.insert(input.stop_actions , input.stop_run)
+
+            else
+                table.insert(input.stop_actions , input.show_statistics)
             end
 
-            table.insert(input.start_actions, input.start_run)
-
-            table.insert(input.stop_actions, input.show_statistics)
-            table.insert(input.stop_actions, input.stop_run)
-
-            function input.start_run() for _, a in pairs(input.start_actions) do a() end end
-            function input.stop_run () for _, a in pairs(input.stop_actions ) do a() end end
-
-            callback.register('start_run', input.start_run)
-            callback.register('stop_run' , input.stop_run )
+            callback.register('start_run', function() for _, a in pairs(input.start_actions) do a() end end)
+            callback.register('stop_run' , function() for _, a in pairs(input.stop_actions ) do a() end ctx.show_statistics() end)
 
         end
 
@@ -335,7 +451,7 @@ end
 
 if texconfig and not texlua then
 
-    if not luatex then luatex = { } end
+    luatex = luatex or { }
 
     luatex.variablenames = {
         'main_memory', 'extra_mem_bot', 'extra_mem_top',
@@ -344,7 +460,7 @@ if texconfig and not texlua then
         'hash_extra', 'max_strings', 'pool_free', 'pool_size', 'string_vacancies',
         'obj_tab_size', 'pdf_mem_size', 'dest_names_size',
         'nest_size', 'param_size', 'save_size', 'stack_size',
-        'trie_size', 'hyph_size',
+        'trie_size', 'hyph_size', 'max_in_open',
         'ocp_stack_size', 'ocp_list_size', 'ocp_buf_size'
     }
 
@@ -371,6 +487,7 @@ if texconfig and not texlua then
     end
 
     texconfig.max_print_line = 100000
+    texconfig.max_in_open    = 127
 
 end
 
